@@ -38,7 +38,6 @@ msm <- function(formula, subject=NULL, data=list(), qmatrix, gen.inits=FALSE,
         emodel <- msm.form.emodel(ematrix, econstraint, initprobs, est.initprobs, qmodel)
     }
     else emodel <- list(misc=FALSE, npars=0, ndpars=0)
-    if (emodel$npars==0) emodel <- list(misc=FALSE, npars=0, ndpars=0) # user supplied degenerate ematrix with no misclassification
 
 ### GENERAL HIDDEN MARKOV MODEL
     if (!is.null(hmodel)) {
@@ -60,8 +59,11 @@ msm <- function(formula, subject=NULL, data=list(), qmatrix, gen.inits=FALSE,
     if (emodel$misc) {
         hmodel <- msm.emodel2hmodel(emodel, qmodel)
     }
-    else emodel <- list(misc=FALSE, npars=0, ndpars=0, nipars=0, nicoveffs=0)
-
+    else {
+        emodel <- list(misc=FALSE, npars=0, ndpars=0, nipars=0, nicoveffs=0)
+        hmodel$ematrix <- FALSE
+    }
+    
 ### EXACT DEATH TIMES. Logical values allowed for backwards compatibility (TRUE means final state has exact death time, FALSE means no states with exact death times)
     if (!is.null(deathexact)) death <- deathexact
     dmodel <- msm.form.dmodel(death, qmodel, hmodel)  # returns death, ndeath,
@@ -104,8 +106,19 @@ msm <- function(formula, subject=NULL, data=list(), qmatrix, gen.inits=FALSE,
     temp[["na.action"]] <- na.pass # run na.action later so we can pass aux info to it
     temp[["data"]] <- data
     mf <- eval(temp, parent.frame())
+    
     ## remember user-specified names for later (e.g. bootstrap/cross validation)
-    attr(mf, "usernames") <- c(state=all.vars(formula[[2]]), time=all.vars(formula[[3]]), subject=as.character(temp$subject), obstype=as.character(substitute(obstype)), obstrue=as.character(temp$obstrue))
+    usernames <- c(state=all.vars(formula[[2]]), time=all.vars(formula[[3]]), subject=as.character(temp$subject), obstype=as.character(substitute(obstype)), obstrue=as.character(temp$obstrue))
+    attr(mf, "usernames") <- usernames
+
+    ## handle matrices in state outcome constructed in formula with cbind()
+    indx <- match(c("formula", "data"), names(call), nomatch = 0)
+    temp <- call[c(1, indx)]
+    temp[[1]] <- as.name("model.frame")
+    mfst <- eval(temp, parent.frame())
+    if (is.matrix(mfst[[1]]) && !is.matrix(mf$"(state)"))
+        mf$"(state)" <- mfst[[1]]
+
     if (is.factor(mf$"(state)")){
         if (!all(grepl("^[[:digit:]]+$", as.character(mf$"(state)"))))
             stop("state variable should be numeric or a factor with ordinal numbers as levels")
@@ -138,6 +151,7 @@ msm <- function(formula, subject=NULL, data=list(), qmatrix, gen.inits=FALSE,
         mf <- na.fail.msmdata(mf)
     else stop ("na.action should be \"na.omit\" or \"na.fail\"")
     attr(mf, "npts") <- length(unique(mf$"(subject)"))
+    attr(mf, "ntrans") <- nrow(mf) - attr(mf, "npts")
 
 ### UTILITY FOR PIECEWISE-CONSTANT INTENSITIES.  Insert censored
     ## observations into the model frame, add a factor "timeperiod" to
@@ -149,7 +163,7 @@ msm <- function(formula, subject=NULL, data=list(), qmatrix, gen.inits=FALSE,
             mf <- tdmodel$mf; covariates <- tdmodel$covariates; cmodel <- tdmodel$cmodel
             pci <- tdmodel$tcut # was returned in msm object
         } else {pci <- NULL; mf$"(pci.imp)" <- 0}
-    }
+    } else tdmodel <- NULL
 
 ### CALCULATE COVARIATE MEANS from data by transition, including means of 0/1 contrasts.
     forms <- c(covariates, misccovariates, hcovariates, initcovariates) # covariates may have been updated to include timeperiod for pci
@@ -183,7 +197,7 @@ msm <- function(formula, subject=NULL, data=list(), qmatrix, gen.inits=FALSE,
 
 ### MODEL FOR COVARIATES ON INTENSITIES
     if (!is.null(covlist)) {
-        cri <- msm.form.cri(covlist, qmodel, mf, mm.cov)
+        cri <- msm.form.cri(covlist, qmodel, mf, mm.cov, tdmodel)
     } else cri <- NULL
     qcmodel <-
         if (ncol(mm.cov) > 1)
@@ -376,6 +390,7 @@ msm.form.qmodel <- function(qmatrix, qconstraint=NULL, analyticp=TRUE, use.expm=
     imatrix <- ifelse(qmatrix > 0, 1, 0)
     inits <- t(qmatrix)[t(imatrix)==1]
     npars <- sum(imatrix)
+    if (npars==0) stop("qmatrix contains all zeroes off the diagonal, so no transitions are permitted in the model")
     ## for phase-type models, leave processing qconstraint until after phased Q matrix has been formed in msm.phase2qmodel
     if (!is.null(qconstraint) && is.null(phase.states)) { 
         if (!is.numeric(qconstraint)) stop("qconstraint should be numeric")
@@ -460,14 +475,19 @@ msm.form.emodel <- function(ematrix, econstraint=NULL, initprobs=NULL, est.initp
     else
         constr <- seq(length=npars)
     ndpars <- if(npars>0) max(constr) else 0
-    emodel <- list(misc=TRUE, npars=npars, nstates=nstates, imatrix=imatrix, ematrix=ematrix, inits=inits,
-                   constr=constr, ndpars=ndpars, nipars=nipars, initprobs=initprobs, est.initprobs=est.initprobs)
+
+    nomisc <- isTRUE(all.equal(as.vector(diag(ematrix)),rep(1,nrow(ematrix)))) # degenerate ematrix with no misclassification: all 1 on diagonal
+    if (nomisc)
+        emodel <- list(misc=FALSE, npars=0, ndpars=0)
+    else                                     
+        emodel <- list(misc=TRUE, npars=npars, nstates=nstates, imatrix=imatrix, ematrix=ematrix, inits=inits,
+                       constr=constr, ndpars=ndpars, nipars=nipars, initprobs=initprobs, est.initprobs=est.initprobs)
     class(emodel) <- "msmemodel"
     emodel
 }
 
 ### Check elements of state vector. For simple models and misc models specified with ematrix
-### No check is performed for hidden models
+### Only check performed for hidden models is that data and model dimensions match
 
 msm.check.state <- function(nstates, state, censor, hmodel)
 {
@@ -477,7 +497,12 @@ msm.check.state <- function(nstates, state, censor, hmodel)
             pl2 <- if (max(hmodel$nout) > 1) "s" else ""
             if ((ncol(state) != max(hmodel$nout)) && (max(hmodel$nout) > 1))
                 stop(sprintf("outcome matrix in data has %d column%s, but outcome models have a maximum of %d dimension%s", ncol(state), pl1, max(hmodel$nout), pl2))
+        } else {
+            if (max(hmodel$nout) > 1) {
+                stop("Only one column in state outcome data, but multivariate hidden model supplied")
+            }
         }
+
     }  else { 
         states <- c(1:nstates, censor)
         state <- na.omit(state) # NOTE added in 1.4
@@ -487,7 +512,8 @@ msm.check.state <- function(nstates, state, censor, hmodel)
             if (length(setdiff(unique(state), states)) > 0)
                 stop("State vector contains elements not in ",statelist)
             miss.state <- setdiff(states, unique(state))
-            if (length(miss.state) > 0)
+            ## Don't do this for misclassification models: it's OK to have particular state not observed by chance
+            if (length(miss.state) > 0 && (!hmodel$hidden || !hmodel$ematrix))
                 warning("State vector doesn't contain observations of ",paste(miss.state, collapse=","))
         }
     }
@@ -515,7 +541,7 @@ msm.check.times <- function(time, subject, state=NULL)
         badlist <- paste(badsubjs, collapse=", ")
         plural <- if (length(badsubjs)==1) "" else "s"
         has <-  if (length(badsubjs)==1) "has" else "have"
-        warning ("Subject", plural, " ", badlist, andothers, " only ", has, " one complete observation")
+        warning ("Subject", plural, " ", badlist, andothers, " only ", has, " one complete observation, which doesn't give any information")
     }
 ### Check if observations within a subject are adjacent
     ind <- tapply(seq_along(subj.num), subj.num, length)
@@ -566,8 +592,10 @@ msm.form.obstype <- function(mf, obstype, dmodel, exacttimes)
         obstype <- rep(2, nrow(mf))
     else {
         obstype <- rep(1, nrow(mf))
-        if (dmodel$ndeath > 0)
-            obstype[mf$"(state)" %in% dmodel$obs] <- 3
+        if (dmodel$ndeath > 0){
+            dobs <- if (is.matrix(mf$"(state)")) apply(mf$"(state)", 1, function(x) any(x %in% dmodel$obs)) else (mf$"(state)" %in% dmodel$obs)
+            obstype[dobs] <- 3
+        }
     }
     obstype
 }
@@ -575,6 +603,9 @@ msm.form.obstype <- function(mf, obstype, dmodel, exacttimes)
 ### On exit, obstrue will contain the true state (if known) or 0 (if unknown)
 ### Any NAs should be replaced by 0 - logically if you don't know whether the state is known or not, that means you don't know the state
 
+### TODO For cases where true state is known, 
+### distinguish between cases where the state data contains the true state or a misclassified version of the true state
+
 msm.form.obstrue <- function(mf, hmodel, cmodel) {
     obstrue <- mf$"(obstrue)"
     if (!is.null(obstrue)) {
@@ -585,7 +616,7 @@ msm.form.obstrue <- function(mf, hmodel, cmodel) {
         else if (!is.numeric(obstrue) && !is.logical(obstrue)) stop("obstrue should be logical or numeric")
         else {
             if (is.logical(obstrue) || (all(na.omit(obstrue) %in% 0:1) && !any(is.na(obstrue)))){
-                ## obstrue is an indicator: actual state is supplied in the outcome vector
+                ## obstrue is an indicator.  Actual state should then be supplied in the outcome vector
                 ## (typically misclassification models)
                 ## interpret presence of NAs as indicating true state supplied here
                 if (!is.null(ncol(mf$"(state)")) && ncol(mf$"(state)") > 1)
@@ -597,6 +628,21 @@ msm.form.obstrue <- function(mf, hmodel, cmodel) {
                     stop("Interpreting \"obstrue\" as containing true states, but it contains values not in 0,1,...,", hmodel$nstates)
                 }
                 obstrue[is.na(obstrue)] <- 0 # true state assumed unknown if NA
+
+                ## If misclassification model specified through "ematrix", interpret the state data as the true state.
+
+                ## If specified through hmodel (including hmmCat), interpret the state as a misclassified/HMM outcome generated conditionally on the true state.
+
+                ## Though that doesn't allow a mixture of true/misclassified states in the same model
+
+                ## If we know the true state, we might sometimes want
+                ## to model a misclassified version of it to
+                ## strengthen estimation of misclassification probs.
+
+                ## In theory we could put NA in the state then to
+                ## indicate that true state is passed through from
+                ## obstrue, and misclassified version isn't known.
+                ## But don't do this until someone asks for it.
             }
         }
     }
@@ -741,7 +787,7 @@ msm.form.covmodel <- function(mf,
         msm.check.constraint(constraint, mm)
         constr <- inits <- numeric()
         maxc <- 0
-        for (i in seq(along=covlabels)){
+        for (i in seq_along(covlabels)){
             ## build complete vectorised list of constraints for covariates in covariates statement
             ## so. e.g. constraints = (x1=c(3,3,4,4,5), x2 = (0.1,0.2,0.3,0.4,0.4))
             ##     turns into constr = c(1,1,2,2,3,4,5,6,7,7) with seven distinct covariate effects
@@ -763,7 +809,7 @@ msm.form.covmodel <- function(mf,
     inits <- numeric()
     if (!is.null(covinits))
         msm.check.covinits(covinits, covlabels)
-    for (i in seq(along=covlabels)) {
+    for (i in seq_along(covlabels)) {
         if (!is.null(covinits) && is.element(covlabels[i], names(covinits))) {
             thisinit <- covinits[[covlabels[i]]]
             if (!is.numeric(thisinit)) {
@@ -808,7 +854,7 @@ msm.form.covmodel.byrate <- function(mf, mm,
     ## Convert short form constraints to long form
     msm.check.constraint(constraint, mm)
     constr <- inits <- numeric()
-    for (i in seq(along=covs)){
+    for (i in seq_along(covs)){
         if (covs[i] %in% names(constraint)){
             if (length(constraint[[covs[i]]]) != sum(cri[,i]))
                 stop("\"",covs[i],"\" constraint of length ",
@@ -821,7 +867,7 @@ msm.form.covmodel.byrate <- function(mf, mm,
     }
     ## convert short to long initial values in the same way
     if (!is.null(covinits)) msm.check.covinits(covinits, covs)
-    for (i in seq(along=covs)) {
+    for (i in seq_along(covs)) {
         if (!is.null(covinits) && (covs[i] %in% names(covinits))) {
             if (!is.numeric(covinits[[covs[i]]])) {
                 warning("initial values for covariates should be numeric, ignoring")
@@ -851,22 +897,25 @@ msm.form.dmodel <- function(death, qmodel, hmodel)
         states <- nstates
     else if (is.logical(death) && death==FALSE)
         states <- numeric(0) ## Will be changed to -1 when passing to C
-    else if (!is.numeric(death)) stop("Death states indicator must be numeric")
+    else if (!is.numeric(death)) stop("Exact death states indicator must be numeric")
     else if (length(setdiff(death, 1:nstates)) > 0)
-        stop("Death states indicator contains states not in ",statelist)
+        stop("Exact death states indicator contains states not in ",statelist)
     else states <- death
     ndeath <- length(states)
     if (hmodel$hidden) {
         ## Form death state info from hmmIdent parameters.
         ## Special observations in outcome data which denote death states
         ## are given as the parameter to hmmIdent()
-        if (!all(hmodel$models[states] == match("identity", .msm.HMODELS)))
-            stop("Death states should have the identity hidden distribution hmmIdent()")
-        obs <- ifelse(hmodel$npars[states]>0, hmodel$pars[hmodel$parstate %in% states], states)
+        mods <- if(is.matrix(hmodel$models)) hmodel$models[1,] else hmodel$models
+        if (!all(mods[states] == match("identity", .msm.HMODELS)))
+            stop("States specified in \"deathexact\" should have the identity hidden distribution hmmIdent()")
+        obs <- ifelse(hmodel$npars[states]>0, 
+                      hmodel$pars[hmodel$parstate %in% states],
+                      states)
     }
     else obs <- states
     if (any (states %in% transient.msm(qmatrix=qmodel$qmatrix)))
-        stop("Not all the \"death\" states are absorbing states")
+        stop("Not all the states specified in \"deathexact\" are absorbing")
     list(ndeath=ndeath, states=states, obs=obs)
 }
 
@@ -914,37 +963,54 @@ msm.form.cmodel <- function(censor=NULL, censor.states=NULL, qmatrix)
 }
 
 ### Transform set of sets of probs {prs} to {log(prs/pr1)}
-msm.mnlogit.transform <- function(pars, plabs, states){
+msm.mnlogit.transform <- function(pars, hmodel){
     res <- pars
+    plabs <- hmodel$plabs
+    states <- hmodel$parstate
+    outcomes <- hmodel$parout
     if (any(plabs=="p")) {
-        whichst <- match(states[plabs == "p"], unique(states[plabs == "p"]))
-        for (i in unique(whichst)) # recalculate baseline prob if necessary, e.g. if constraints applied
-            res[plabs=="pbase"][i] <- 1 - sum(res[plabs=="p"][whichst==i])
-        res[plabs == "p"] <- log(pars[plabs=="p"] / res[plabs=="pbase"][whichst])
+        for (i in unique(states)) {
+            for (j in unique(outcomes)) {
+                pfree <- which(plabs=="p" & states==i & outcomes==j)
+                pbase <- which(plabs=="pbase" & states==i & outcomes==j)
+                ## recalculate baseline prob if necessary, e.g. if constraints applied
+                res[pbase] <- 1 - sum(res[pfree])
+                res[pfree] <- log(res[pfree] / res[pbase])
+            }
+        }
     }
     res
 }
 
 ### Transform set of sets of murs = {log(prs/pr1)} to probs {prs}
 ### ie psum = sum(exp(mus)),  pr1 = 1 / (1 + psum),  prs = exp(mus) / (1 + psum)
-msm.mninvlogit.transform <- function(pars, plabs, states){
+msm.mninvlogit.transform <- function(pars, hmodel) {
     res <- pars
-    if (any(plabs=="p")) {
+    plabs <- hmodel$plabs
+    states <- hmodel$parstate
+    outcomes <- hmodel$parout 
+    if (any(plabs=="p")) { # p's are unconstrained probabilities 
+        ## indicator for which p's belong to which state 
         whichst <- match(states[plabs=="p"], unique(states[plabs=="p"]))
-        if (is.matrix(pars)) {# will be used when applying covariates
-            for (i in unique(whichst)) {
-                psum <- colSums(exp(pars[which(plabs=="p")[whichst==i],,drop=FALSE]))
-                res[which(plabs=="pbase")[i],] <- 1 / (1 + psum)
-                res[which(plabs=="p")[whichst==i],] <-
-                    exp(pars[plabs=="p",,drop=FALSE][whichst==i,]) /
-                        rep(1 + psum, each=sum(whichst==i))
+        ## indicator for which p's belong to which outcome in multivariate HMMs
+        whichout <- match(outcomes[plabs=="p"], unique(outcomes[plabs=="p"]))
+        for (i in unique(whichst)) {
+            for (j in unique(whichout)) { 
+                pfree <- which(plabs=="p")[whichst==i & whichout==j] 
+                pbase <- which(plabs=="pbase" & states==i & outcomes==j)
+
+                if (is.matrix(pars)) {# will be used when applying covariates
+                    psum <- colSums(exp(pars[pfree,,drop=FALSE]))
+                    res[pbase,] <- 1 / (1 + psum)
+                    res[pfree,] <- exp(pars[pfree,,drop=FALSE]) /
+                      rep(1 + psum, each=sum(whichst==i & whichout==j))
+                } else {
+                    psum <- sum(exp(pars[pfree]))
+                    res[pbase] <- 1 / (1 + psum)
+                    res[pfree] <- exp(pars[pfree]) / (1 + psum)
+                }
             }
         }
-        else {
-            psum <- tapply(exp(pars[plabs=="p"]), states[plabs=="p"], sum)
-            res[plabs=="pbase"][unique(states[plabs=="p"])] <- 1 / (1 + psum) # don't transform pbase if no p's for this state, i.e. if no/perfect misclassification
-            res[plabs=="p"] <- exp(pars[plabs=="p"]) / (1 + psum[whichst])
-        }
     }
     res
 }
@@ -956,7 +1022,7 @@ msm.transform <- function(pars, hmodel, ranges){
     pars <- glogit(pars, ranges[,"lower"], ranges[,"upper"])
     hpinds <- which(!(labs %in% c("qbase","qcov","hcov","initpbase","initp","initp0","initpcov")))
     hpars <- pars[hpinds]
-    hpars <- msm.mnlogit.transform(hpars, hmodel$plabs, hmodel$parstate)
+    hpars <- msm.mnlogit.transform(hpars, hmodel)
     pars[hpinds] <- hpars
     pars[labs=="initp"] <- log(pars[labs=="initp"] / pars[labs=="initpbase"])
     pars
@@ -969,7 +1035,7 @@ msm.inv.transform <- function(pars, hmodel, ranges){
     pars <- gexpit(pars, ranges[,"lower"], ranges[,"upper"])
     hpinds <- which(!(labs %in% c("qbase","qcov","hcov","initp","initp0","initpcov")))
     hpars <- pars[hpinds]
-    hpars <- msm.mninvlogit.transform(hpars, hmodel$plabs, hmodel$parstate)
+    hpars <- msm.mninvlogit.transform(hpars, hmodel)
     pars[hpinds] <- hpars
     ep <- exp(pars[labs=="initp"])
     pars[labs=="initp"] <- ep / (1 + sum(ep))
@@ -1094,10 +1160,10 @@ msm.rep.constraints <- function(pars, # transformed pars
     }
     ## ...then replicate them between states, on pr scale, so that constraint applies
     ## to pr not log(pr/pbase). After, transform back
-    pars[p$hmmpars] <- msm.mninvlogit.transform(pars[p$hmmpars], hmodel$plabs, hmodel$parstate)
+    pars[p$hmmpars] <- msm.mninvlogit.transform(pars[p$hmmpars], hmodel)
     plabs <- plabs[!duplicated(abs(p$constr))][abs(p$constr)]
     pars <- pars[!duplicated(abs(p$constr))][abs(p$constr)]*sign(p$constr)
-    pars[p$hmmpars] <- msm.mnlogit.transform(pars[p$hmmpars], hmodel$plabs, hmodel$parstate)
+    pars[p$hmmpars] <- msm.mnlogit.transform(pars[p$hmmpars], hmodel)
     names(pars) <- plabs
     pars
 }
@@ -1191,7 +1257,7 @@ msm.add.hmmcovs <- function(hmodel, pars, mml){
     for (i in seq_along(hpinds)){
         hpars[i,] <- gexpit(hpars[i,], hmodel$ranges[i,"lower"], hmodel$ranges[i,"upper"])
     }
-    hpars <- msm.mninvlogit.transform(hpars, hmodel$plabs, hmodel$parstate)
+    hpars <- msm.mninvlogit.transform(hpars, hmodel)
     hpars
 }
 
@@ -1407,7 +1473,7 @@ Ccall.msm <- function(params, do.what="lik", msmdata, qmodel, qcmodel, cmodel, h
                "(state)" = as.double(t(mf$"(state)")), 
                "(obstype)" = as.integer(mf$"(obstype)"),
               "(obstrue)" = as.integer(mf$"(obstrue)"), "(pcomb)" = as.integer(mf$"(pcomb)"))
-   auxdata <- list(nagg=as.integer(nagg),n=as.integer(nrow(mf)),npts=as.integer(attr(mf,"npts")),
+    auxdata <- list(nagg=as.integer(nagg),n=as.integer(nrow(mf)),npts=as.integer(attr(mf,"npts")),
                    ntrans=as.integer(ntrans), npcombs=as.integer(npcombs),
                    nout = as.integer(if(is.null(ncol(mf$"(state)"))) 1 else ncol(mf$"(state)")),
                    nliks=as.integer(get("nliks",msm.globals)),firstobs=as.integer(firstobs))
@@ -1420,7 +1486,8 @@ Ccall.msm <- function(params, do.what="lik", msmdata, qmodel, qcmodel, cmodel, h
    hmodel <- list(hidden=as.integer(hmodel$hidden), mv=as.integer(hmodel$mv),
                   models=as.integer(hmodel$models),
                   totpars=as.integer(hmodel$totpars), firstpar=as.integer(hmodel$firstpar),
-                  npars=as.integer(hmodel$npars), nopt=as.integer(hmodel$nopt))
+                  npars=as.integer(hmodel$npars), nopt=as.integer(hmodel$nopt),
+                  ematrix=as.integer(hmodel$ematrix))
     pars <- list(Q=as.double(Q),DQ=as.double(DQ),H=as.double(H),DH=as.double(DH),
                 initprobs=as.double(initprobs),nopt=as.integer(nopt))
     .Call("msmCEntry",  as.integer(match(do.what, .msm.CTASKS) - 1),
@@ -1467,11 +1534,11 @@ msm.form.output <- function(x, whichp)
         else mat[mat==1] <- Inf ## if no parinds are "p", then there are off-diag 1s in ematrix
         mat <- t(mat)
         dimnames(mat) <- dimnames(model$imatrix)
+        fixed <- array(FALSE, dim=dim(model$imatrix))
         if (p$foundse && !p$fixed){
             intenscov <- p$covmat[parinds, parinds]
             intensse <- sqrt(diag(as.matrix(intenscov)))
             semat <- lmat <- umat <- t(model$imatrix)
-            fixed <- array(FALSE, dim=dim(model$imatrix))
             if (any(parinds)){
                 semat[t(model$imatrix)==1] <- intensse
                 lmat[t(model$imatrix)==1] <- p$ci[parinds,1]
@@ -1482,7 +1549,8 @@ msm.form.output <- function(x, whichp)
             semat <- t(semat); lmat <- t(lmat); umat <- t(umat); fixed <- t(fixed)
             diag(semat) <- diag(lmat) <- diag(umat) <- 0
             for (i in 1:nrow(fixed)){
-                fixed[i,i] <- all(fixed[i,-i][model$imatrix[i,-i]==1])
+                foff <- fixed[i,-i][model$imatrix[i,-i]==1]
+                fixed[i,i] <- (length(foff)>1) && all(foff)
             }
             if (whichp=="misc")
                 fixed[which(x$hmodel$model==match("identity", .msm.HMODELS)),] <- TRUE
@@ -1490,7 +1558,7 @@ msm.form.output <- function(x, whichp)
             dimnames(semat)  <- dimnames(mat)
         }
         else {
-            semat <- lmat <- umat <- fixed <- NULL
+            semat <- lmat <- umat <- NULL
         }
         Matrices[[matrixname]] <- mat
         MatricesSE[[matrixname]] <- semat
@@ -1718,7 +1786,7 @@ msm.check.covlist <- function(covlist, qemodel) {
         badnums <- paste(badnums, collapse=",")
         stop("Name", plural1, " ", badnames, " of \"covariates\" formula", plural2, " ", badnums, " not in format \"number-number\"")
     }
-    for (i in seq(along=covlist))
+    for (i in seq_along(covlist))
         if (!inherits(covlist[[i]], "formula"))
             stop("\"covariates\" should be a formula or list of formulae")
     trans <- sapply(strsplit(names(covlist), "-"), as.numeric)
@@ -1734,12 +1802,19 @@ msm.check.covlist <- function(covlist, qemodel) {
 ## Form indicator matrix for effects that will be fixed to zero when
 ## "covariates" specified as a list of transition-specific formulae
 
-msm.form.cri <- function(covlist, qmodel, mf, mm) {
+msm.form.cri <- function(covlist, qmodel, mf, mm, tdmodel) {
+
     imat <- t(qmodel$imatrix) # order named transitions / misclassifications by row
     tnames <- paste(col(imat)[imat==1],row(imat)[imat==1],sep="-")
     covlabs <- colnames(mm)[-1]
     npars <- qmodel$npars
     cri <- matrix(0, nrow=npars, ncol=length(covlabs), dimnames = list(tnames, covlabs))
+    ## time effects specified through "pci" will always be applied to all transitions
+    if (!is.null(tdmodel)){
+        grep("timeperiod\\[.+,.+\\)", covlabs)
+        tdcovs <- grep("timeperiod\\[.+,.+\\)", covlabs)
+        cri[,tdcovs] <- 1
+    }
     sorti <- function(x) {
         ## converts, e.g. c("b:a:c","d:f","f:e") to c("a:b:c", "d:f", "e:f")
         sapply(lapply(strsplit(x, ":"), sort), paste, collapse=":")
@@ -1780,7 +1855,7 @@ na.find.msmdata <- function(object, ...) {
     lastobs <- !duplicated(subj, fromLast=TRUE)
     nm <- names(object)
     omit <- FALSE
-    for (j in seq(along=object)) {
+    for (j in seq_along(object)) {
         ## Drop all NAs in time, subject as usual
         if (nm[j] %in% c("(time)", "(subject)"))
             omit <- omit | is.na(object[[j]])
